@@ -34,6 +34,7 @@ import {
 
 import {
   retrieve_config_file,
+  update_thoth_config_on_disk,
   lock_requirements_with_thoth,
   lock_requirements_with_pipenv
 } from '../thoth';
@@ -42,6 +43,7 @@ import {
   get_kernel_name,
   set_requirements,
   set_requirement_lock,
+  set_resolution_engine,
   set_thoth_configuration,
   delete_key_from_notebook_metadata
 } from "../notebook"
@@ -80,6 +82,7 @@ export interface IState {
   initial_packages: { [ name: string ]: string },
   deleted_packages: { [ name: string ]: string },
   requirements: Requirements,
+  thoth_config: ThothConfig,
   error_msg: string
 }
 
@@ -122,6 +125,20 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
           requires: { python_version: get_python_version( this.props.panel ) },
           sources: [new Source()]
         },
+        thoth_config: {
+          host: "khemenu.thoth-station.ninja",
+          tls_verify: false,
+          requirements_format: "pipenv",
+          runtime_environments: [{
+            name: "ubi:8",
+            operating_system: {
+              name: "ubi",
+              version: "8",
+            },
+            python_version: "3.8",
+            recommendation_type: "latest"
+        }]
+        },
         error_msg: undefined
       }
     }
@@ -138,6 +155,7 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
       deleted_packages: { [ name: string ]: string },
       requirements: Requirements,
       kernel_name: string,
+      thoth_config: ThothConfig,
       error_msg?: string,
     ) {
 
@@ -158,6 +176,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
 
       _.set(new_state, "kernel_name", kernel_name)
 
+      _.set(new_state, "thoth_config", thoth_config)
+
       _.set(new_state, "error_msg", error_msg)
 
       console.log("new", new_state)
@@ -218,7 +238,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
         this.state.installed_packages,
         this.state.deleted_packages,
         this.state.requirements,
-        this.state.kernel_name
+        this.state.kernel_name,
+        this.state.thoth_config
       )
     }
 
@@ -242,7 +263,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
         this.state.installed_packages,
         this.state.deleted_packages,
         this.state.requirements,
-        this.state.kernel_name
+        this.state.kernel_name,
+        this.state.thoth_config
       )
 
     }
@@ -269,7 +291,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
         this.state.installed_packages,
         this.state.deleted_packages,
         this.state.requirements,
-        this.state.kernel_name
+        this.state.kernel_name,
+        this.state.thoth_config
       )
 
     }
@@ -300,7 +323,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
         this.state.installed_packages,
         deleted_packages,
         this.state.requirements,
-        this.state.kernel_name
+        this.state.kernel_name,
+        this.state.thoth_config
       )
 
     }
@@ -331,7 +355,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
         this.state.installed_packages,
         deleted_packages,
         this.state.requirements,
-        this.state.kernel_name
+        this.state.kernel_name,
+        this.state.thoth_config
       )
     }
 
@@ -363,7 +388,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
         this.state.installed_packages,
         this.state.deleted_packages,
         this.state.requirements,
-        this.state.kernel_name
+        this.state.kernel_name,
+        this.state.thoth_config
       )
 
     }
@@ -431,16 +457,17 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
             this.state.installed_packages,
             {},
             emptyRequirements,
-            this.state.kernel_name
+            this.state.kernel_name,
+            this.state.thoth_config
           )
         }
 
       }
 
       // Check if there are packages saved, otherwise go to failed notification message
-      if ( _.size(total_packages ) > 0 ) {
+      if ( _.size( total_packages ) > 0 ) {
 
-        if ( _.isEqual(total_packages, this.state.installed_packages) ){
+        if ( _.isEqual( total_packages, this.state.installed_packages) ){
 
           var sameRequirements: Requirements = {
             packages: total_packages,
@@ -461,10 +488,10 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
             this.state.installed_packages,
             this.state.deleted_packages,
             this.state.requirements,
-            this.state.kernel_name
+            this.state.kernel_name,
+            this.state.thoth_config
           )
           return
-
         }
 
         else {
@@ -492,7 +519,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
             this.state.installed_packages,
             this.state.deleted_packages,
             finalRequirements,
-            this.state.kernel_name
+            this.state.kernel_name,
+            this.state.thoth_config
           )
 
           return
@@ -519,7 +547,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
           this.state.installed_packages,
           this.state.deleted_packages,
           this.state.requirements,
-          this.state.kernel_name
+          this.state.kernel_name,
+          this.state.thoth_config
         )
         return
       }
@@ -539,7 +568,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
             this.state.initial_packages,
             this.state.deleted_packages,
             this.state.requirements,
-            this.state.kernel_name
+            this.state.kernel_name,
+            this.state.thoth_config
           )
 
           return
@@ -556,6 +586,7 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
           this.state.deleted_packages,
           this.state.requirements,
           this.state.kernel_name,
+          this.state.thoth_config,
           "Error install dependencies in the new virtual environment, please contact Thoth team."
         )
       }
@@ -594,7 +625,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
             this.state.initial_packages,
             this.state.deleted_packages,
             this.state.requirements,
-            this.state.kernel_name
+            this.state.kernel_name,
+            this.state.thoth_config
           )
 
           return
@@ -611,6 +643,7 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
             this.state.deleted_packages,
             this.state.requirements,
             this.state.kernel_name,
+            this.state.thoth_config,
             "Error setting new environment in a jupyter kernel, please contact Thoth team."
           )
 
@@ -628,33 +661,15 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
         this.state.installed_packages,
         this.state.deleted_packages,
         this.state.requirements,
-        this.state.kernel_name
+        this.state.kernel_name,
+        this.state.thoth_config
       )
 
-      const thothConfig: ThothConfig = await this.createConfig();
+      const thothConfig: ThothConfig = this.state.thoth_config;
+      console.log("thoth config to be submitted", JSON.stringify(thothConfig));
 
-      // If the endpoint cannot be reached or there are issues with thamos config creation
-
-      if (_.isUndefined(thothConfig)) {
-
-        console.log("Thoth config is undefined")
-
-        this.lock_using_pipenv()
-      }
-
-      const runtime_environments: RuntimeEnvornment[] = thothConfig.runtime_environments
-
-      const runtime_environment: RuntimeEnvornment = runtime_environments[0]
-
-      // TODO: Assign user recommendation type to all runtime environments in thoth config?
-      _.set(runtime_environment, "recommendation_type", this.state.recommendation_type)
-      _.set(runtime_environments, 0, runtime_environment)
-      _.set(thothConfig, "runtime_environments", runtime_environments)
-
-      console.log("thoth config submitted", JSON.stringify(thothConfig));
-
-      const notebookMetadataRequirements = this.state.requirements;
-      console.log("Requirements submitted", JSON.stringify(notebookMetadataRequirements));
+      const notebookMetadataRequirements: Requirements = this.state.requirements;
+      console.log("Requirements to be submitted", JSON.stringify(notebookMetadataRequirements));
 
       try {
 
@@ -666,28 +681,58 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
         console.log("Advise received", advise);
 
         if ( advise.error == false ) {
-          // Set requirements in notebook;
-          set_requirements( this.props.panel , advise.requirements )
-          set_requirement_lock( this.props.panel , advise.requirement_lock )
-          set_thoth_configuration( this.props.panel , thothConfig )
+          if ( _.isEmpty( advise.requirements.packages ) == false) {
 
-          await this.store_dependencies_on_disk(
-            advise.requirements,
-            advise.requirement_lock,
-            'overlays',
-            false
-          )
+             // Set requirements, requirements lock and thoth config in notebook;
+            set_requirements( this.props.panel , advise.requirements )
+            set_requirement_lock( this.props.panel , advise.requirement_lock )
+            set_thoth_configuration( this.props.panel , thothConfig )
+            set_resolution_engine( this.props.panel , "thoth" )
 
-          this.changeUIstate(
-            "installing_requirements",
-            {},
-            this.state.initial_packages,
-            this.state.installed_packages,
-            {},
-            advise.requirements,
-            this.state.kernel_name
-          )
+            // Save all changes to disk.
+            this.props.panel.context.save()
+            try {
+              await this.store_dependencies_on_disk(
+                advise.requirements,
+                advise.requirement_lock,
+                'overlays',
+                false
+              )
+            } catch ( error ) {
 
+              console.log("Error storing dependencies in overlays", error)
+
+            }
+
+            try {
+              await update_thoth_config_on_disk(
+                this.state.thoth_config.runtime_environments[0],
+                true
+              )
+            } catch ( error ) {
+              console.log("Error updating thoth config on disk", error)
+            }
+
+            this.changeUIstate(
+              "installing_requirements",
+              {},
+              this.state.initial_packages,
+              this.state.installed_packages,
+              {},
+              advise.requirements,
+              this.state.kernel_name,
+              this.state.thoth_config
+            )
+
+            return
+
+          }
+          else {
+
+            console.log("Advise requirements packages received is empty", advise.requirements);
+
+            this.lock_using_pipenv()
+          }
         }
         else {
 
@@ -712,7 +757,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
         this.state.installed_packages,
         this.state.deleted_packages,
         this.state.requirements,
-        this.state.kernel_name
+        this.state.kernel_name,
+        this.state.thoth_config
       )
 
       const notebookMetadataRequirements = this.state.requirements;
@@ -729,14 +775,35 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
 
         if ( result.error == false ) {
 
+          set_requirements( this.props.panel , notebookMetadataRequirements )
           set_requirement_lock( this.props.panel , result.requirements_lock )
+          set_resolution_engine( this.props.panel , "pipenv" )
 
-          await this.store_dependencies_on_disk(
-            notebookMetadataRequirements,
-            result.requirements_lock,
-            'overlays',
-            false
-          )
+          // Save all changes to disk.
+          this.props.panel.context.save()
+
+          try {
+            await this.store_dependencies_on_disk(
+              notebookMetadataRequirements,
+              result.requirements_lock,
+              'overlays',
+              false
+            )
+
+          } catch ( error ) {
+
+            console.log("Error storing dependencies in overlays", error)
+
+          }
+
+          try {
+            await update_thoth_config_on_disk(
+              this.state.thoth_config.runtime_environments[0],
+              true
+            )
+          } catch ( error ) {
+            console.log("Error updating thoth config on disk", error)
+          }
 
           this.changeUIstate(
             "installing_requirements",
@@ -745,7 +812,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
             this.state.installed_packages,
             {},
             notebookMetadataRequirements,
-            this.state.kernel_name
+            this.state.kernel_name,
+            this.state.thoth_config
           )
 
           return
@@ -761,6 +829,7 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
             this.state.deleted_packages,
             this.state.requirements,
             this.state.kernel_name,
+            this.state.thoth_config,
             "No resolution engine was able to install dependendices, please contact Thoth team."
           )
 
@@ -779,6 +848,7 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
           this.state.deleted_packages,
           this.state.requirements,
           this.state.kernel_name,
+          this.state.thoth_config,
           "No resolution engine was able to install dependendices, please contact Thoth team."
         )
         return
@@ -788,6 +858,33 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
 
     async onStart() {
 
+        // Load thoth_config from notebook metadata, if any, otherwise get default one
+        var thoth_config: ThothConfig = this.props.initial_config_file
+
+        if ( thoth_config == null ) {
+          // No Thoth config found in notebook metadata, create default one
+          console.log("No initial thoth config found, creating default one")
+          var thoth_config: ThothConfig = await this.createConfig();
+        }
+
+        // If the endpoint cannot be reached or there are issues with thamos config creation
+
+        if (_.isUndefined(thoth_config)) {
+          console.warn("Thoth config is undefined")
+          var thoth_config: ThothConfig = this.state.thoth_config;
+        }
+
+        const runtime_environments: RuntimeEnvornment[] = thoth_config.runtime_environments
+        const runtime_environment: RuntimeEnvornment = runtime_environments[0]
+
+        // TODO: Assign user recommendation type to all runtime environments in thoth config?
+        _.set(runtime_environment, "name", this.state.kernel_name)
+        _.set(runtime_environment, "recommendation_type", this.state.recommendation_type)
+        _.set(runtime_environments, 0, runtime_environment)
+        _.set(thoth_config, "runtime_environments", runtime_environments)
+
+        console.log("initial thoth config", thoth_config)
+
         // Load requirements from notebook metadata, if any, otherwise receive default ones
         var initial_requirements: Requirements = this.props.initial_requirements
         console.log("initial requirements", initial_requirements)
@@ -802,7 +899,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
             this.state.installed_packages,
             this.state.deleted_packages,
             initial_requirements,
-            this.state.kernel_name
+            this.state.kernel_name,
+            thoth_config
           )
           return
         }
@@ -822,7 +920,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
             this.state.installed_packages,
             this.state.deleted_packages,
             initial_requirements,
-            this.state.kernel_name
+            this.state.kernel_name,
+            thoth_config
           )
           return
         }
@@ -879,7 +978,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
               initial_packages,
               this.state.deleted_packages,
               initial_requirements,
-              kernel_name
+              kernel_name,
+              thoth_config
               )
 
             return
@@ -894,7 +994,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
               initial_installed_packages,
               this.state.deleted_packages,
               initial_requirements,
-              kernel_name
+              kernel_name,
+              thoth_config
               )
             return
           }
@@ -908,7 +1009,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
             initial_installed_packages,
             this.state.deleted_packages,
             initial_requirements,
-            kernel_name
+            kernel_name,
+            thoth_config
             )
           return
         }
@@ -949,7 +1051,6 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
     }
 
     async createConfig() {
-      // TODO: Use config created automatically by thamos??
       const config_file = await retrieve_config_file( this.state.kernel_name);
       console.log("Config file", config_file);
 
@@ -1205,7 +1306,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
                   this.state.installed_packages,
                   this.state.deleted_packages,
                   this.state.requirements,
-                  this.state.kernel_name
+                  this.state.kernel_name,
+                  this.state.thoth_config,
                 )
                 }
                 >
@@ -1252,7 +1354,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
                       this.state.installed_packages,
                       this.state.deleted_packages,
                       this.state.requirements,
-                      this.state.kernel_name
+                      this.state.kernel_name,
+                      this.state.thoth_config,
                     )
                     }
                     >
@@ -1308,7 +1411,8 @@ export class DependenciesManagementUI extends React.Component<IProps, IState> {
                         this.state.installed_packages,
                         this.state.deleted_packages,
                         this.state.requirements,
-                        this.state.kernel_name
+                        this.state.kernel_name,
+                        this.state.thoth_config,
                       )
                       }
                       >
