@@ -78,11 +78,9 @@ export function delete_key_from_notebook_metadata( notebook: NotebookPanel , key
 
 export function get_requirements( notebook: NotebookPanel ):  Promise<Requirements> {
     return new Promise( async ( resolve, reject ) => {
-
         try {
-            const retrieved = notebook.model.metadata.get("requirements")
 
-            if (typeof retrieved == "undefined") {
+            if ( notebook.model.metadata.has("requirements") == false ) {
                 console.log("requirements key is not in notebook metadata! Initialize requirements for user...")
                 const python_packages: { [ name: string ]: string } = {}
                 const requires = { python_version: get_python_version( notebook ) }
@@ -98,6 +96,8 @@ export function get_requirements( notebook: NotebookPanel ):  Promise<Requiremen
 
             }
 
+            const retrieved = notebook.model.metadata.get("requirements")
+
             let notebookMetadataRequirements: Requirements = JSON.parse(retrieved.toString())
 
             if (_.size(notebookMetadataRequirements) == 0) {
@@ -132,10 +132,10 @@ export function get_requirements( notebook: NotebookPanel ):  Promise<Requiremen
  */
 
 export function set_requirements( notebook: NotebookPanel, requirements: Requirements ): void {
-    const metadata = notebook.model.metadata
+    const notebook_metadata = notebook.model.metadata
 
-    if ( metadata.has("requirements") == false ) {
-        metadata.set('requirements', JSON.stringify(requirements) )
+    if ( notebook_metadata.has("requirements") == false ) {
+        notebook_metadata.set('requirements', JSON.stringify(requirements) )
         console.log( "Notebook requirements have been set successfully." )
 
     } else {
@@ -143,14 +143,13 @@ export function set_requirements( notebook: NotebookPanel, requirements: Require
 
         // update the notebook metadata
 
-        console.log()
-        // Insert empty requirements if no packages are present.
-        if  (typeof requirements.packages === 'undefined')  {
-            metadata.set('requirements', {} )
-            return
+        // Fail if no packages are present are not in requirements.
+        if  ( _.isEmpty(requirements.packages) == true)  {
+            console.log( "Notebook requirements packages is empty..." )
+            throw new Error( `Packages in notebook requirements is empty: '${ requirements }' `, )
         }
 
-        metadata.set('requirements', JSON.stringify(requirements) )
+        notebook_metadata.set('requirements', JSON.stringify(requirements) )
         return
     }
 }
@@ -161,16 +160,16 @@ export function set_requirements( notebook: NotebookPanel, requirements: Require
 
 export function get_requirement_lock( notebook: NotebookPanel ): Promise<RequirementsLock|null> {
     return new Promise( async ( resolve, reject ) => {
+        const notebook_metadata = notebook.model.metadata
 
         try {
-            const retrieved = notebook.model.metadata.get("requirements_lock")
-
-            if (typeof retrieved == "undefined") {
+            if ( notebook_metadata.has("requirements_lock") == false ) {
                 console.log("requirements_lock key is not in notebook metadata!")
                 resolve( null )
             }
 
-            var notebookMetadataRequirementsLock: RequirementsLock = JSON.parse(retrieved.toString())
+            const requirements_lock = notebook_metadata.get("requirements_lock")
+            var notebookMetadataRequirementsLock: RequirementsLock = JSON.parse(requirements_lock.toString())
 
             console.log("requirements_lock key is in notebook metadata!")
             resolve ( notebookMetadataRequirementsLock )
@@ -186,15 +185,15 @@ export function get_requirement_lock( notebook: NotebookPanel ): Promise<Require
  */
 
 export function set_requirement_lock( notebook: NotebookPanel, requirements_lock: RequirementsLock ): void {
-    const metadata = notebook.model.metadata
+    const notebook_metadata = notebook.model.metadata
 
-    if ( metadata.has("requirements_lock") == false ) {
-        metadata.set('requirements_lock', JSON.stringify(requirements_lock) )
+    if ( notebook_metadata.has("requirements_lock") == false ) {
+        notebook_metadata.set('requirements_lock', JSON.stringify(requirements_lock) )
 
     } else {
         console.debug( "Notebook requirement_lock already exist. Updating." )
         // update the notebook metadata
-        metadata.set('requirements_lock', JSON.stringify(requirements_lock) )
+        notebook_metadata.set('requirements_lock', JSON.stringify(requirements_lock) )
     }
 
     console.log( "Notebook requirements_lock have been set successfully." )
@@ -204,18 +203,29 @@ export function set_requirement_lock( notebook: NotebookPanel, requirements_lock
 /**
  * Function: Get Thoth configuration from notebook metadata.
  */
-export function get_thoth_configuration( notebook: NotebookPanel ): ThothConfig | null {
+export function get_thoth_configuration( notebook: NotebookPanel ): Promise<ThothConfig|null> {
 
-    let retrieved = notebook.model.metadata.get("thoth_config")
+    return new Promise( async ( resolve, reject ) => {
+        const notebook_metadata = notebook.model.metadata
 
-    if (retrieved == null) {
-        console.log("thoth_config key is not in notebook metadata!")
-        return null
+        try {
+            if ( notebook_metadata.has("thoth_config") == false ) {
+                console.log("thoth_config key is not in notebook metadata!")
+                resolve( null )
+            }
+
+            const thoth_config = notebook_metadata.get("thoth_config")
+            var notebookMetadataThothConfig: ThothConfig = JSON.parse(thoth_config.toString())
+
+            console.log("thoth_config key is in notebook metadata!")
+            resolve ( notebookMetadataThothConfig )
+
+        } catch ( err ) {
+            reject( err )
+        }
+    })
     }
 
-    return  JSON.parse(JSON.stringify(retrieved))
-}
-
 /**
  * Function: Set Thoth config file requirements into notebook metadata.
  */
@@ -234,3 +244,23 @@ export function set_thoth_configuration( notebook: NotebookPanel, config_file: T
 
     console.log( "Notebook Thoth config have been set successfully." )
   }
+
+
+/**
+ * Function: Set resolution engine name used into notebook metadata.
+ */
+
+export function set_resolution_engine( notebook: NotebookPanel, dependency_resolution_engine: string ): void {
+    const metadata = notebook.model.metadata
+
+    if ( metadata.has("dependency_resolution_engine") == false ) {
+        metadata.set('dependency_resolution_engine', dependency_resolution_engine )
+
+    } else {
+        console.debug( "Dependency resolution engine used for requirements already exist. Updating." )
+        // update the notebook metadata
+        metadata.set('dependency_resolution_engine', dependency_resolution_engine )
+    }
+
+    console.log( "Dependency resolution engine used for requirements have been set successfully." )
+  }
